local addonName, addon = ...

local L = setmetatable({ }, { __index = function(self, key)
	self[key] = key
	return key
end })
addon.L = L

local function OnMouseWheel(self, delta)
	if not (self.type == CONTROLTYPE_SLIDER and self.cvar) then
		self = AudioOptionsSoundPanelMasterVolume
	end
	local value = tonumber(GetCVar(self.cvar))
	if value then
		local adjustedValue = value + delta * (IsModifierKeyDown() and 0.01 or 0.05)
		if adjustedValue < 0 then
			adjustedValue = 0
		elseif adjustedValue > 1 then
			adjustedValue = 1
		end
		if adjustedValue ~= value then
			SetCVar(self.cvar, tostring(adjustedValue))
			BlizzardOptionsPanel_Slider_Refresh(self)
		end
	end
end

local dataObj = LibStub('LibDataBroker-1.1'):NewDataObject(addonName, {
	type = 'data source',
	label = SOUND_LABEL,
	icon = ([[Interface\AddOns\%s\Sound%sabled]]):format(addonName, GetCVarBool('Sound_EnableAllSound') and "En" or "Dis"),
	text = ("%d%%"):format(((tonumber(GetCVar('Sound_MasterVolume')) or 1) + 0.005) * 100),

	OnClick = function(self, button)
		if button == 'LeftButton' then
			SetCVar('Sound_EnableAllSound', GetCVarBool('Sound_EnableAllSound') and 0 or 1, 'ENABLE_SOUND')
			if AudioOptionsSoundPanel:IsVisible() then
				BlizzardOptionsPanel_CheckButton_Refresh(AudioOptionsSoundPanelEnableSound)
			end
		elseif button == 'RightButton' then
			if AudioOptionsSoundPanel:IsVisible() then
				HideUIPanel(VideoOptionsFrame)
			else
--				OptionsFrame_OpenToCategory(VideoOptionsFrame, AudioOptionsSoundPanel)
				OptionsFrame_OpenToCategory(VideoOptionsFrame, AudioOptionsSoundPanel.name)	-- Bliz bug work around
			end
		end
	end,

	OnMouseWheel = OnMouseWheel,

	OnTooltipShow = function(tooltip)
		local red, green, blue = NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b
		tooltip:SetText(SOUND_LABEL, 1, 1, 1)
		tooltip:AddDoubleLine(L["Left-Click"], L["Toggle Mute"], 0.15, 0.85, 0.15, red, green, blue, true)
		tooltip:AddDoubleLine(L["Mouse Wheel"], L["Adjust Volume"], 0.15, 0.85, 0.15, red, green, blue, true)
		tooltip:AddDoubleLine(L["Right-Click"], L["Toggle Configuration"], 0.15, 0.85, 0.15, red, green, blue, true)
	end
})

hooksecurefunc('SetCVar', function(cVar, value)
	if cVar == 'Sound_EnableAllSound' then
		dataObj.icon = ([[Interface\AddOns\%s\Sound%sabled]]):format(addonName, GetCVarBool(cVar) and "En" or "Dis")
	elseif cVar == 'Sound_MasterVolume' then
		dataObj.text = ("%d%%"):format(((tonumber(GetCVar(cVar)) or 1) + 0.005) * 100)
	end
end)

for cVar, data in pairs(SoundPanelOptions) do
	if data.minValue == 0 and data.maxValue == 1 then
		local option = data.text:match("(%a+)_%a+")
		if option then
			local slider = _G[("AudioOptionsSoundPanel%s%sVolume"):format(option:sub(1, 1), option:sub(2):lower())]
			if slider and slider.cvar == cVar and not slider:GetScript('OnMouseWheel') then
				slider:SetScript('OnMouseWheel', OnMouseWheel)
				slider:EnableMouseWheel(true)
			end
		end		
	end
end
